<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Permohonan;
use App\Models\Pencatatan;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Services\PencatatanService;
use Illuminate\Support\Facades\Redirect;


class PencatatanController extends Controller
{
    private $pencatatanService;

    public function __construct()
    {
        $this->pencatatanService = new PencatatanService();
    }
    
    public function create($id_permohonan)
    {
        $permohonan = Permohonan::where('id', '=', $id_permohonan)->first();
        $this->pencatatanService->createPencatatan($permohonan);
        
        return redirect()->action('Admin\PermohonanController@index');
    }
    
    public function update($id,Request $request)
    {
		$pencatatan = Pencatatan::where('id', '=', $id)->first();

		$errorMessages = [];
		if ($request->session()->has('errorMessages')) {
			$errorMessages = $request->session()->get('errorMessages');
			$permohonan->fill($request->session()->get('permohonanData'));
			$request->session()->forget(['errorMessages', 'permohonanData']);
		}
		return view('pencatatan.update', [
			'pencatatan' => $pencatatan,
			'errorMessages' => $errorMessages
		]);
    }
    
    public function postUpdate(Request $request)
    {
        $user = Auth::user();

        $rules = [
            'noKtpPria' => 'required|digits:16',
            'namaPria' => 'required',
            'noKtpWanita' => 'required|digits:16',
            'namaWanita' => 'required',
            'statusPria' => 'required',
            'statusWanita' => 'required',
            'istriPriaSebelumnya' => 'required_if:statusPria,1',
            'suamiWanitaSebelumnya' => 'required_if:statusWanita,1'
        ];

        $messages = [
            'required'=> 'Field :attribute tidak boleh kosong',
            'digits' => 'Field :attribute harus berupa angka dan memiliki panjang 16 digit'
        ];

        $validator = Validator::make($request->input(), $rules, $messages);

        if ($validator->fails()) {
            $request->session()->set('errorMessages', $validator->errors()->all());
            $request->session()->set('permohonanData', $request->input());
            return redirect('permohonan/update');
        } else {
            $this->permohonanService->updatePermohonan($user, $request->input());
            return redirect('permohonan/update');
        }
    }
}
