<?php

namespace App\Http\Controllers;

use App\Auth;
use App\Http\Controllers\Core\CommonUserController;
use App\Models\Permohonan;
use App\Models\Penduduk;
use App\Services\PermohonanService;
use App\UserKependudukan;
use Illuminate\Http\Request;
use Validator;

class PermohonanController extends CommonUserController
{
    private $permohonanService;

    public function __construct(Request $request)
    {
        $this->permohonanService = new PermohonanService();
    }
    
    public function getToken(){
		if (isset($_GET['token'])) {
			return $_GET['token'];
		}else return NULL;

	}
    
    public function getNIKTerkait($token){
        
        $nik = null;
        
        if ($token){
			
			$userKependudukan = UserKependudukan::where('token','=',$token)->first();
			
			if ($userKependudukan)
			{
				$nik = $userKependudukan->nik;
			}
			
		}
		
		return $nik;
	}
	
	public function getPermohonanFromNIK($nik){

		
		$permohonan = Permohonan::where('noKtpPria','=',$nik)
					->orWhere('noKtpWanita','=',$nik)
					->first();
					
		return $permohonan;
	}
    
    public function index(Request $request)
    {
		
        $token = $this->getToken();
		
		$nik = $this->getNIKTerkait($token);
		
		if ($nik){
			
			$permohonan = $this->getPermohonanFromNIK($nik);		
			
			return view('permohonan.index', [ 'permohonan' => $permohonan, 'token' => $token ]);

		}else{
			
			if (!$this->authenticate($request)) {
				return redirect()->action('HomeController@login');
			}

			$user = $this->user($request);
			
			return view('permohonan.index', [
				'permohonan' => $user->permohonan, 'token' => null
			]);
		}
    }
    
    public function create(Request $request)
    {

        $token = $this->getToken();
        		
		$nik = $this->getNIKTerkait($token);
		
		if ($nik){
			
			$permohonan = $this->getPermohonanFromNIK($nik);
			
			$this->permohonanService->createPermohonanNIK($nik);
			
			return redirect()->action('PermohonanController@update',["token" => $token]);
			
		}else{

			if (!$this->authenticate($request)) {
				return redirect()->action('HomeController@login');
			}

			$user = $this->user($request);
			$this->permohonanService->createPermohonan($user);
			
			return redirect()->action('PermohonanController@update');
			
		}
    }
    
    public function update(Request $request)
    {
        
        $token = $this->getToken();
		
		$nik = $this->getNIKTerkait($token);
		
		if ($nik){
			
			$permohonan = $this->getPermohonanFromNIK($nik);
			
			if ($permohonan){

				$errorMessages = [];
				if ($request->session()->has('errorMessages')) {
					$errorMessages = $request->session()->get('errorMessages');
					$permohonan->fill($request->session()->get('permohonanData'));
					$request->session()->forget(['errorMessages', 'permohonanData']);
				}
				return view('permohonan.update', [
					'permohonan' => $permohonan,
					'errorMessages' => $errorMessages,
					'token' => $token
				]);
			}else{
					return redirect()->action('PermohonanController@create');
			}
		}else{
			
			if (!$this->authenticate($request)) {
				return redirect()->action('HomeController@login');
			}

			$user = $this->user($request);
			if ($this->permohonanService->userHavePermohonan($user)) {
				$permohonan = $user->permohonan;

				$errorMessages = [];
				if ($request->session()->has('errorMessages')) {
					$errorMessages = $request->session()->get('errorMessages');
					$permohonan->fill($request->session()->get('permohonanData'));
					$request->session()->forget(['errorMessages', 'permohonanData']);
				}
				return view('permohonan.update', [
					'permohonan' => $permohonan,
					'errorMessages' => $errorMessages,
					'token' => null
				]);
			} else {
				return redirect()->action('PermohonanController@createPermohonan',["token" => $token]);
			}
			
		}
    }
    
		
	public $validateIdentitasMessages;
    public function validateIdentitas($noKtpPria, $noKtpWanita){
		
		$this->validateIdentitasMessages = [];
		
		$valid = true;
		
		if ($noKtpPria != NULL){
			$pendudukPria = Penduduk::where('id','=',$noKtpPria)->first();
			if ($pendudukPria == NULL){
				array_push($this->validateIdentitasMessages, "Penduduk pria tidak ditemukan");
				$valid = false;
			}else{
				if ($pendudukPria->jenis_kelamin=='P'){
					array_push($this->validateIdentitasMessages, "noKtpPria harus diisi dengan NIK dari yang berjenis kelamin pria");
					$valid=false;
				}
			}
		}
		
		if ($noKtpWanita != NULL){
			$pendudukWanita = Penduduk::where('id','=',$noKtpWanita)->first();
			if ($pendudukWanita == NULL){
				array_push($this->validateIdentitasMessages, "Penduduk wanita tidak ditemukan");
				$valid = false;
			}else{
				if ($pendudukWanita->jenis_kelamin=='L'){
					array_push($this->validateIdentitasMessages, "noKtpWanita harus diisi dengan NIK dari yang berjenis kelamin wanita");
					$valid=false;
				}
			}
		}
		
		return $valid;
	}
    
    public function postUpdate(Request $request)
    {
        
        $token = $this->getToken();
		
		$nik = $this->getNIKTerkait($token);
		
		if ($nik==NULL){
			if (!$this->authenticate($request)) {
				return redirect()->action('HomeController@login');
			}

			$user = $this->user($request);
		}

        $rules = [
            'noKtpPria' => 'required|digits:16',
            'namaPria' => 'required',
            'noKtpWanita' => 'required|digits:16',
            'namaWanita' => 'required',
            'statusPria' => 'required',
            'statusWanita' => 'required',
            'istriPriaSebelumnya' => 'required_if:statusPria,1',
            'suamiWanitaSebelumnya' => 'required_if:statusWanita,1'
        ];

        $messages = [
            'required'=> 'Field :attribute tidak boleh kosong',
            'digits' => 'Field :attribute harus berupa angka dan memiliki panjang 16 digit'
        ];

        $validator = Validator::make($request->input(), $rules, $messages);

        if ($validator->fails()) {
            $request->session()->set('errorMessages', $validator->errors()->all());
            $request->session()->set('permohonanData', $request->input());
            return redirect('permohonan/update?token='.$token);
        } else if (!$this->validateIdentitas($request->input()['noKtpPria'],$request->input()['noKtpWanita'])){
            $request->session()->set('errorMessages', $this->validateIdentitasMessages);
            $request->session()->set('permohonanData', $request->input());
		} else {
			if ($nik){
				$this->permohonanService->updatePermohonanNIK($nik, $request->input());
			}else{
				$this->permohonanService->updatePermohonan($user, $request->input());
			}
            return redirect('permohonan/update?token='.$token);
        }
    }

    public function upload(Request $request)
    {
        if (!$this->authenticate($request)) {
            return redirect()->action('HomeController@login');
        }

        $user = $this->user($request);
        $permohonan = $user->permohonan;
        
        return view('permohonan.upload', [
            'permohonan' => $permohonan
        ]);
    }
    
    public function postUploadRt(Request $request)
    {
        if (!$this->authenticate($request)) {
            return redirect()->action('HomeController@login');
        }

        $user = $this->user($request);
        $permohonan = $user->permohonan;
        $uploadPath = public_path(Permohonan::SURAT_RT_DIR);

        if ($request->hasFile('file')) {
            $permohonan->uploadSuratRt = true;
            $permohonan->save();
            $request->file('file')->move($uploadPath, $permohonan->id);
        } else {

        }

        return redirect()->action('PermohonanController@upload');
    }
    
    public function postUploadRw(Request $request)
    {
        
    }
}
